package org.gcube.portlets.user.geoportaldataentry.client.events;

import java.util.List;

import org.gcube.application.geoportalcommon.shared.products.model.RecordDV;
import org.gcube.portlets.user.geoportaldataentry.client.ConstantsGeoPortalDataEntryApp.ACTION_ON_ITEM;

import com.google.gwt.event.shared.GwtEvent;

/**
 * The Class ActionOnItemEvent.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR francesco.mangiacrapa@isti.cnr.it
 * 
 *         Sep 15, 2021
 * @param <T> the generic type
 */
public class ActionOnItemEvent<T extends RecordDV> extends GwtEvent<ActionOnItemEventHandler> {
	public static Type<ActionOnItemEventHandler> TYPE = new Type<ActionOnItemEventHandler>();
	private List<T> selectItems;
	private ACTION_ON_ITEM action;

	/**
	 * Instantiates a new action on item event.
	 *
	 * @param selectItems the select items
	 * @param doAction    the do action
	 */
	public ActionOnItemEvent(List<T> selectItems, ACTION_ON_ITEM doAction) {
		this.selectItems = selectItems;
		this.action = doAction;
	}

	/**
	 * Gets the associated type.
	 *
	 * @return the associated type
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.google.gwt.event.shared.GwtEvent#getAssociatedType()
	 */
	@Override
	public Type<ActionOnItemEventHandler> getAssociatedType() {
		return TYPE;
	}

	/**
	 * Dispatch.
	 *
	 * @param handler the handler
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.google.gwt.event.shared.GwtEvent#dispatch(com.google.gwt.event.shared.
	 * EventHandler)
	 */
	@Override
	protected void dispatch(ActionOnItemEventHandler handler) {
		handler.onDoActionFired(this);
	}

	/**
	 * Gets the select items.
	 *
	 * @return the select items
	 */
	public List<T> getSelectItems() {
		return selectItems;
	}

	/**
	 * Gets the action.
	 *
	 * @return the action
	 */
	public ACTION_ON_ITEM getAction() {
		return action;
	}

	/**
	 * To string.
	 *
	 * @return the string
	 */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("ActionOnItemEvent [selectItems=");
		builder.append(selectItems);
		builder.append(", action=");
		builder.append(action);
		builder.append("]");
		return builder.toString();
	}

}
